function [y, X, f, alpha, t, objective, status, status_text, lp_time, lp_set_time] = gdsp_lp_relax_cplex(W,ubs,lbs,g,H_ubs, H_lbs, S, dist, d_0)
%
% (C)2012-13 Syama Sundar Rangapuram, Thomas Buehler and Matthias Hein 
% Machine Learning Group, Saarland University, Germany
% http://www.ml.uni-saarland.de    
    
    seeds = S;
	n = size(W,1); n_rest = n - length(seeds);    
    ind_rest = setdiff((1:n)', seeds);   
    Wtriu_rest = triu(W(ind_rest, ind_rest) );
    [ixt_rest, jxt_rest, wvalt_rest] = find(Wtriu_rest);
    mhalf_rest = length(ixt_rest);
    
    g_rest = g(ind_rest);
	lbs_rest = lbs-sum(H_lbs(S,:), 1)'; ubs_rest = ubs-sum(H_ubs(S,:), 1)'; H_lbs_rest = H_lbs(ind_rest, :); H_ubs_rest = H_ubs(ind_rest, :);
    assert( sum(ubs_rest<0) ==0, 'The seed set already violates upper bound constraints.');

    mu_S = sum(sum(W(seeds, seeds)));
    nu_S = sum(g(seeds));
    deg_S = sum(W(ind_rest, seeds), 2);
    
    %D = dist; D(dist<=d_0) = 0; D(dist>d_0) = 1; [dix, djx] = find(D);
    dist_rest = dist(ind_rest, ind_rest);
    [Dix_rest, Djx_rest] = find(dist_rest > d_0); 
    
    disp(' ');
	display('*****************************************************   ');
    display(['  Starting the LP']);
    lp_start_time = tic;

    lp_set_time_start = tic;
    display('  Preparing the inputs for cplex... ');
    Abounds1 = [speye(mhalf_rest) sparse(mhalf_rest, n_rest) -1*ones(mhalf_rest,1)];  % alpha_ij <= t <=> alpha_ij - t <= 0 
    bbounds1 = zeros(mhalf_rest,1);
    Abounds2 = [sparse(n_rest, mhalf_rest) speye(n_rest) -1*ones(n_rest,1)];  % f_i <= t <=> f_i - t <= 0
    bbounds2 = zeros(n_rest,1);
    Abounds = [Abounds1; Abounds2]; bbounds = [bbounds1; bbounds2];


    Aedge1 = [speye(mhalf_rest) sparse(1:mhalf_rest, ixt_rest, -1, mhalf_rest, n_rest) zeros(mhalf_rest,1)]; % alpha_ij <= f_i <=> alpha_ij - f_i <= 0
    Aedge2 = [speye(mhalf_rest) sparse(1:mhalf_rest, jxt_rest, -1, mhalf_rest, n_rest) zeros(mhalf_rest,1)]; % alpha_ij <= f_j <=> alpha_ij - f_j <= 0
    Aedge = [Aedge1; Aedge2];
    bedge = zeros(mhalf_rest*2,1);

    Askill_ub = [sparse(length(ubs_rest), mhalf_rest) H_ubs_rest' -ubs_rest];  % H_ubs'*f <= t*ubs <=> H_ubs*f - t*ubs <= 0; H_ubs is nExperts x nSkills matrix
    bskill_ub = zeros(length(ubs_rest),1);
    Askill_lb = [sparse(length(lbs_rest), mhalf_rest) -H_lbs_rest' lbs_rest];  % H_lbs'*f >= t*lbs <=> -H_lbs*f + t*lbs <= 0; H_lbs is nExperts x nSkills matrix
    bskill_lb = zeros(length(lbs_rest),1);

    Adist = [sparse(length(Dix_rest), mhalf_rest) sparse(1:length(Dix_rest), Dix_rest, 1, length(Dix_rest), n_rest) + sparse(1:length(Djx_rest), Djx_rest, 1, length(Djx_rest), n_rest) -ones(length(Dix_rest), 1)]; ...
            % 1_(dist(u, v) > d)) (f_u + f_v) <= t, \forall u, v <=>  y_Dix + y_Djx - t <= 0
    bdist = sparse(length(Dix_rest), 1);
        
    Ascale = zeros(1,mhalf_rest+n_rest+1);  % <g, f> + t* \nu_S = 1 => <g, f> + t* \nu_S <= 1
    Ascale( (mhalf_rest+1):(end-1) ) = g_rest; Ascale(end) = nu_S;
    bscale = 1;
%     Ascale2 = zeros(1, mhalf_rest+n_rest+1);  % <g, f> + t* \nu_S = 1 => <g, f> + t* \nu_S >= 1 => -<g, f> - t* \nu_S <= -1
%     Ascale2( (mhalf_rest+1): (end-1)) = -g_rest; Ascale2(end) = -nu_S;
%     bscale2 = -1;
%     Ascale = [Ascale1; Ascale2]; bscale = [bscale1; bscale2];

    A = [Abounds; Aedge; Askill_lb; Askill_ub; Adist];
    b = [bbounds; bedge; bskill_lb; bskill_ub; bdist];
    
    cost = -[2*wvalt_rest; 2*deg_S; mu_S]; %assoc_S(C) still contains 2 cut(A, S). cut(A, S) gets reduced when you rewrite assoc(C) in terms of cut in the redcued graph!
    lp_set_time = toc(lp_set_time_start);

    display('  Calling the cplex routine');

    % signature: [x, fval, exitflag, output, lambda] = cplexlp (f, Aineq, bineq, Aeq, beq, lb, ub, x0, options)
    [sol_rest, obj, status, output] = cplexlp(cost, A, b, Ascale, bscale, zeros(mhalf_rest+n_rest+1, 1), []); % possible to give a starting point        
    status_text = output.message;

    obj = -obj;
    alpha_rest = sol_rest(1:mhalf_rest);
    f_rest = sol_rest( (mhalf_rest+1) : (end-1));
    t = sol_rest(end);  % just a single variable and has no connection to the given graph

    X_rest = alpha_rest/t; y_rest = f_rest/t; % get the original variables
    y = ones(n, 1); y(ind_rest) = y_rest; % get the solution on the original graph
    %X = sparse(mhalf, 1);  % X = ones on S!
    [ixt, jxt, wvalt] = find(triu(W)); X = sparse(1:length(ixt), 1, min(y(ixt), y(jxt)), length(ixt), 1);
    
    f = t*y;  % optional: get the solution of the transformed problem on the original graph.
    alpha = t*X;

    lp_time = toc(lp_start_time);
    
    
    assert( sum( H_lbs'*y + 1e-6 < lbs  ) == 0 );  % lower bound constraints are satisfied upto a tolerance of 1e-6
    assert( sum( H_ubs'*y - 1e-6 > ubs  ) == 0 );  % upper bound constraints are satisfied upto a tolerance of 1e-6
    
    assert( sum( abs( X_rest - min( y_rest(ixt_rest), y_rest(jxt_rest) ) ) )  <= 1e-6);  % X_ij = min(y_i, y_j) upto a tolerance of 1e-6, \forall (i, j) \in E
    
    display(['  LP found the optimum = ', num2str(obj), ' in ', num2str(lp_time), ' seconds', '. The returned flag is ', num2str(status), ' and status text is ', status_text]);
    objective_rest = (2*wvalt_rest'*X_rest + 2*deg_S'*y_rest + mu_S) /(g_rest'*y_rest + nu_S);
    objective = 2*wvalt'*X /(g'*y);
    
    assert( abs(objective - obj) <= 1e-8 );
    assert( abs(objective_rest - obj) <= 1e-8 );
    display('*****************************************************   ');

end


% cvx_begin
%     variables X(mhalf,1) y(n,1) t
%     maximize ( wvalt'*X  )
%     for i=1:mhalf
%         X(i) <= y(ixt(i));
%         X(i) <= y(jxt(i));        
%     end
%     X >= 0;
%     y >= 0;
%     y <= t;
%     X <= t;
%     t >= 0;
%     H_lbs'*y >= lbs*t;
%     H_ubs'*y <= ubs*t;
%     y'*ones(n,1) == 1;
% 
% cvx_end
% status = cvx_status;
% X = X/t;
% y = y/t;