function [flow,labels] = maxflow(A,T)

%MAXFLOW    Max-flow/min-cut calculation using the
%   Boykov-Kolmogorov's algorithm. Let G=(V,E) be
%   a directed graph with |V| vertices and |E| edges.
%   w:(i,j)-->R, (i,j) in E, a weight function on the graph's
%   edges.
%
%   A (smoothness term) - a sparse matrix of size |V|x|V| containing, where
%   A(i,j) = w(i,j). Typically, for a grid graph, the node 
%   indices would be taken in column-major order, but it is not assumed.
%   T (data term) - a sparse matrix of size |V|x|L|, where |L| is the number
%   of labels (The library currently supports |L|=2 only).
%   T(i,j) = the cost of assigning label j to node i.
%   
%   flow - the calculated maximum flow value
%   labels - a vector of size |V|, where labels(i) is 0 or 1 if
%   node i belongs to S (source) or T (sink) respectively.
%
%   Also refer to the following link for tips on creating large
%   sparse matrices efficiently.
%   In case of a dense adjacency matrix B, simply wrap it in a 
%   sparse structure before passing it to this function:
%   A = sparse(B);
%   [flow,labels] = maxflow(A,T);
%
%   This library can be used for reseatch purposes only, and is
%   supplied with no guarantees.
%
%   (c) 2008 Michael Rubinstein, WDI R&D and IDC
%   $Revision: 140 $
%   $Date: 2008-09-15 15:35:01 -0700 (Mon, 15 Sep 2008) $
%
% Copyright (c) 2008, Michael Rubinstein
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.

[flow,labels] = maxflowmex(A,T);

% release the dll
clear maxflowmex

end